{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE TypeOperators #-}
{-# LANGUAGE DataKinds #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE TemplateHaskell #-}

module ISTEX.Client where

import GHC.Generics
import Data.Aeson
import Control.Applicative ((<|>))

import Servant.API
import Servant.Client
import Data.Proxy (Proxy(..))

import qualified Data.Text as T
import qualified Control.Lens as L

data Author = Author
  {
    _name :: T.Text,
    _affiliations :: [T.Text]
  } deriving (Show, Generic)
L.makeLenses ''Author
instance FromJSON Author where
  parseJSON (Object o) =
    Author <$> (o .: "name") <*> (o .: "affiliations" <|> pure [])

data Document = Document
  {
    _id :: T.Text,
    _title :: Maybe T.Text,
    _authors :: [Author],
    _abstract :: Maybe T.Text,
    _publicationDate :: Maybe Int
  } deriving (Show, Generic)
L.makeLenses ''Document

instance FromJSON Document where
  parseJSON (Object o) =
    Document <$>
    (o .: "id")
    <*> (o .:? "title")
    <*> (o .: "author" <|> pure [])
    <*> (o .:? "abstract")
    <*> do pPubDate <- (o .:? "publicationDate")
           return $ (read . T.unpack) <$> pPubDate

data Documents = Documents
  {
    _total :: Int,
    _hits :: [Document]
  } deriving (Show, Generic)
L.makeLenses ''Documents
instance FromJSON Documents where
  parseJSON (Object o) =
    Documents <$> (o .: "total") <*> (o .: "hits")

type ISTEXAPI = Search

type Search =
  QueryParam "output" T.Text
  :> QueryParam "size" Int
  :> QueryParam "q" T.Text
  :> Get '[JSON] Documents

istexProxy :: Proxy (ISTEXAPI)
istexProxy = Proxy

search :: Maybe T.Text -> Maybe Int -> Maybe T.Text -> ClientM Documents
search = client istexProxy
